@@ -30,6 +30,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <ctype.h>
 #include <sys/types.h>
 #include <signal.h>
 #include <libgnome/gnome-macros.h>
@@ -38,18 +39,18 @@
 #include <libgnomevfs/gnome-vfs-monitor.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs-xfer.h>
 #include <gbf/gbf-project.h>
 #include <gbf/glue-plugin.h>
 #include "gbf-i18n.h"
 #include "gbf-am-project.h"
-#include "gbf-am-build.h"
 #include "gbf-am-config.h"
 #include "gbf-am-properties.h"
 
 #define ENABLE_DEBUG
 
 #ifdef ENABLE_DEBUG
-#define DEBUG(x) (x)
+#define DEBUG(x) x
 #else
 #define DEBUG(x)
 #endif
@@ -131,6 +132,9 @@ struct _GbfAmProjectParseData {
 
 	/* hash table to keep track of possibly removed nodes */
 	GHashTable         *nodes;
+	
+	/* Errors accumulation */
+	GString            *error;
 };
 
 
@@ -183,11 +187,6 @@ static GbfProject *parent_class;
    Private prototypes
    ---------------------------------------------------------------------- */
 
-static gboolean        queue_check                  (GbfAmProject      *project);
-static void            queue_push_op                (GbfAmProject      *project,
-						     GbfAmProjectOp    *op);
-static void            queue_destroy                (GbfAmProject      *project);
-
 static gboolean        uri_is_equal                 (const gchar       *uri1,
 						     const gchar       *uri2);
 static gboolean        uri_is_parent                (const gchar       *parent_uri,
@@ -231,7 +230,8 @@ static GError         *parse_errors                 (GbfAmProject      *project,
 static gboolean        parse_output_xml             (GbfAmProject      *project,
 						     gchar             *xml,
 						     gint               length,
-						     GSList           **change_set);
+						     GSList           **change_set,
+						     gchar            **error_message);
 
 static void            spawn_shutdown               (GbfAmSpawnData    *data);
 static void            spawn_data_destroy           (GbfAmSpawnData    *data);
@@ -282,112 +282,6 @@ static void            gbf_am_project_get_property  (GObject           *object,
  * - Operations are added to the head and retrieved from the tail
  */
 
-static gboolean
-queue_check (GbfAmProject *project)
-{
-	GbfAmProjectOp *op;
-
-	g_return_val_if_fail (project != NULL, FALSE);
-	g_return_val_if_fail (GBF_IS_AM_PROJECT (project), FALSE);
-
-	g_assert (project->queue_ops != NULL);
-	
-	if (g_queue_is_empty (project->queue_ops)) {
-		project->queue_handler_tag = 0;
-		return FALSE;
-	}
-
-	op = g_queue_pop_tail (project->queue_ops);
-
-	switch (op->type) {
-		case BUILD: 
-		{
-			gchar *project_root = uri_to_path (project->project_root_uri);
-						
-			/* FIXME: gbf_build_run can be simpler */
-			gbf_build_run (project, op->build_id,
-				       project_root,
-				       project->callbacks);
-
-			g_free (op->build_id);
-			g_free (project_root);
-			
-			break;
-		}
-	}
-
-	g_free (op);
-
-	/* FIXME: this matches the old behavior, but it's wrong */
-	if (g_queue_is_empty (project->queue_ops)) {
-		project->queue_handler_tag = 0;
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-static void
-queue_push_op (GbfAmProject *project, GbfAmProjectOp *op)
-{
-	g_return_if_fail (project != NULL);
-	g_return_if_fail (GBF_IS_AM_PROJECT (project));
-	g_return_if_fail (op != NULL);
-	
-	g_assert (project->queue_ops != NULL);
-	
-	g_queue_push_head (project->queue_ops, op);
-
-	if (project->queue_handler_tag == 0)
-		project->queue_handler_tag = g_idle_add ((GSourceFunc) queue_check, project);
-}
-
-static void
-queue_destroy (GbfAmProject *project)
-{
-	g_return_if_fail (project != NULL);
-	g_return_if_fail (GBF_IS_AM_PROJECT (project));
-	
-	if (project->queue_ops) {
-		GbfAmProjectOp *op;
-		while ((op = g_queue_pop_tail (project->queue_ops)) != NULL)
-			g_free (op);
-		g_queue_free (project->queue_ops);
-		project->queue_ops = NULL;
-	}
-
-	if (project->queue_handler_tag) {
-		g_source_remove (project->queue_handler_tag);
-		project->queue_handler_tag = 0;
-	}
-}
-
-/*
- * Callbacks bookkeepping -----------------------------
- */
-
-static int
-find_callback (gconstpointer a, gconstpointer b)
-{
-	GbfAmBuildCallback *cb = (GbfAmBuildCallback *)a;
-	if (cb->callback == b)
-		return 0;
-	else
-		return -1;
-}
-
-static void
-callbacks_destroy (GbfAmProject *project)
-{
-	g_return_if_fail (project != NULL);
-	g_return_if_fail (GBF_IS_AM_PROJECT (project));
-
-	/* FIXME */
-	g_list_free (project->callbacks);
-	project->callbacks = NULL;
-}
-
-
 /*
  * URI and path manipulation functions -----------------------------
  */
@@ -1625,6 +1519,20 @@ sax_end_element (void *ctx, const xmlChar *name)
 	}
 }
 
+static void
+sax_error (void *ctx, const char *format, ...)
+{
+	va_list args;
+	char buffer[256];
+	
+	va_start (args, format);
+	vsnprintf (buffer, 256, format, args);
+	va_end (args);
+	
+	GbfAmProjectParseData *data = ctx;
+	g_string_append (data->error, buffer);
+}
+
 #undef PARSER_ASSERT
 
 static void 
@@ -1649,7 +1557,8 @@ static gboolean
 parse_output_xml (GbfAmProject *project,
 		  gchar        *xml_text,
 		  gint          length,
-		  GSList      **change_set)
+		  GSList      **change_set,
+		  gchar **      error_message)
 {
 	xmlSAXHandler handler;
 	GbfAmProjectParseData data;
@@ -1658,6 +1567,7 @@ parse_output_xml (GbfAmProject *project,
 	memset (&handler, 0, sizeof (xmlSAXHandler));
 	handler.startElement = sax_start_element;
 	handler.endElement = sax_end_element;
+	handler.error = sax_error;
 	handler.initialized = 0;
 	
 	data.state = PARSE_INITIAL;
@@ -1672,14 +1582,26 @@ parse_output_xml (GbfAmProject *project,
 	data.change_set = NULL;
 	data.nodes = g_hash_table_new (g_direct_hash, g_direct_equal);
 	data.compute_change_set = (change_set != NULL);
-
+	data.error = g_string_new (NULL);
+	
 	xmlSubstituteEntitiesDefault (TRUE);
 
 	/* Parse document */
 	retval = xmlSAXUserParseMemory (&handler, &data, xml_text, length);
 
+	if (retval != 0)
+	{
+		if (error_message) {
+			if (data.error->len > 0)
+				*error_message = g_strdup (data.error->str);
+		}
+	}
+	g_string_free (data.error, TRUE);
+	
 	if (data.state != PARSE_DONE)
+	{
 		retval = -1;
+	}
 
 	/* construct the change set */
 	if (retval >= 0 && data.compute_change_set) {
@@ -1700,7 +1622,7 @@ parse_output_xml (GbfAmProject *project,
 	}
 	g_free (data.param_key);
 	
-	return (retval >= 0);
+	return (retval == 0);
 }
 
 
@@ -1920,7 +1842,7 @@ spawn_write_child (GIOChannel *ioc, GIOCondition condition, gpointer user_data)
 
 		switch (status) {
 		    case G_IO_STATUS_NORMAL:
-			    DEBUG (g_message ("wrote %d bytes", bytes_written));
+			    DEBUG (g_message ("wrote %" G_GSIZE_FORMAT " bytes", bytes_written));
 			    
 			    if (data->input.length < data->input.size) {
 				    /* don't remove the source */
@@ -2243,10 +2165,21 @@ project_reload (GbfAmProject *project, GError **err)
 		}
 		
 		if (data->output.length > 0) {
+			gchar *xml_error = NULL;
 			retval = parse_output_xml (project,
 						   data->output.buffer,
 						   data->output.length,
-						   NULL);
+						   NULL, &xml_error);
+			if (err != NULL && *err == NULL &&
+			    !retval && xml_error) {
+				/* xml parse error */
+				g_set_error (err,
+					     GBF_PROJECT_ERROR,
+					     GBF_PROJECT_ERROR_GENERAL_FAILURE,
+					     "XML parse error: %s",
+					     xml_error);
+			}
+			g_free (xml_error);
 		} else {
 			/* FIXME: generate some kind of error here */
 			g_warning ("Child process returned no data");
@@ -2302,11 +2235,23 @@ project_update (GbfAmProject *project,
 		}
 		
 		if (data->output.length > 0) {
+			gchar *xml_error = NULL;
 			/* process the xml output for changed groups */
 			retval = parse_output_xml (project,
 						   data->output.buffer,
 						   data->output.length,
-						   change_set);
+						   change_set, &xml_error);
+			if (err != NULL && *err == NULL &&
+			    !retval && xml_error) {
+				/* xml parse error */
+				g_set_error (err,
+					     GBF_PROJECT_ERROR,
+					     GBF_PROJECT_ERROR_GENERAL_FAILURE,
+					     "XML parse error: %s",
+					     xml_error);
+			}
+			g_free (xml_error);
+			
 			/* FIXME: emit this only if the project has indeed changed */
 			g_signal_emit_by_name (G_OBJECT (project), "project-updated");
 		}
@@ -2669,43 +2614,6 @@ impl_load (GbfProject  *_project,
 	}
 }
 
-static void
-impl_build (GbfProject  *_project,
-	    const gchar *id,
-	    GError     **error)
-{
-	GbfAmProject *project;
-	GbfAmProjectOp *op;
-
-	g_return_if_fail (GBF_IS_AM_PROJECT (_project));
-
-	project = GBF_AM_PROJECT (_project);
-	op = g_new0 (GbfAmProjectOp, 1);
-	op->project = project;
-	op->type = BUILD;
-
-	op->build_id = g_strdup (id);
-	queue_push_op (project, op);
-}
-
-static void
-impl_clean (GbfProject *_project,
-	    GError    **error)
-{
-	GbfAmProject *project;
-	GbfAmProjectOp *op;
-
-	g_return_if_fail (GBF_IS_AM_PROJECT (_project));
-
-	project = GBF_AM_PROJECT (_project);
-	op = g_new0 (GbfAmProjectOp, 1);
-	op->project = project;
-	op->type = BUILD;
-	op->build_id = g_strdup (GBF_AM_BUILD_ID_CLEAN);
-
-	queue_push_op (project, op);
-}
-
 static gboolean
 file_exists (const gchar *path, const gchar *filename)
 {
@@ -2758,111 +2666,14 @@ impl_refresh (GbfProject *_project,
 		g_signal_emit_by_name (G_OBJECT (project), "project-updated");
 }
 
-
-static void
-foreach_build_target (gpointer key, gpointer value, gpointer data)
+static GbfProjectCapabilities
+impl_get_capabilities (GbfProject *_project, GError    **error)
 {
-	GList **targets = data;
-	GbfAmNode *node;
-	GbfBuildTarget *target;
-
-	node = GBF_AM_NODE ((GNode *)value);
-
-	if ( 0 == strcmp (node->detail, "program") ||
-	     0 == strcmp (node->detail, "static_lib") ||
-	     0 == strcmp (node->detail, "shared_lib") ) {
-		target = g_new0 (GbfBuildTarget, 1);
-		target->id = g_strdup_printf ("USER:%s", (gchar *) key);
-		target->label = g_strdup (node->name);
-		target->description = g_strdup_printf (_("Build specific target: %s"), node->name);
-
-		*targets = g_list_append (*targets, target);
-	}
-}
-
-
-static GList *
-impl_get_build_targets (GbfProject *_project,
-			GError    **error)
-{
-	GbfAmProject *project;
-	GbfBuildTarget *target;
-	GList *targets = NULL;
-
-	g_return_val_if_fail (GBF_IS_AM_PROJECT (_project), NULL);
-
-	project = GBF_AM_PROJECT (_project);
-
-	/* FIXME: these targets should be gotten from the Makefile.am files */
-	/* "autogen" */
-	target = g_new0 (GbfBuildTarget, 1);
-	target->id = g_strdup (GBF_AM_BUILD_ID_AUTOGEN);
-	target->label = g_strdup (_("Generate"));
-	target->description = g_strdup (_("Generate Makefile"));
-	targets = g_list_append (targets, target);
-
-	/* "configure" */
-	target = g_new0 (GbfBuildTarget, 1);
-	target->id = g_strdup (GBF_AM_BUILD_ID_CONFIGURE);
-	target->label = g_strdup (_("Configure"));
-	target->description = g_strdup (_("Configure the project"));
-	targets = g_list_append (targets, target);
-
-	/* "make all" */
-	target = g_new0 (GbfBuildTarget, 1);
-	target->id = g_strdup (GBF_AM_BUILD_ID_ALL);
-	target->label = g_strdup (_("All"));
-	target->description = g_strdup (_("Build the entire project"));
-	targets = g_list_append (targets, target);
-
-	/* "make install" */
-	target = g_new0 (GbfBuildTarget, 1);
-	target->id = g_strdup (GBF_AM_BUILD_ID_INSTALL);
-	target->label = g_strdup (_("Install"));
-	target->description = g_strdup (_("Build the entire project and install it"));
-	targets = g_list_append (targets, target);
-
-	/* other build targets */
-	g_hash_table_foreach (project->targets, foreach_build_target, &targets);
-
-	return targets;
-}
-
-static void
-impl_add_build_callback (GbfProject      *_project,
-			 GbfBuildCallback callback,
-			 gpointer         user_data,
-			 GError         **error)
-{
-	GbfAmProject *project;
-	GbfAmBuildCallback *cb;
-
-	g_return_if_fail (GBF_IS_AM_PROJECT (_project));
-
-	project = GBF_AM_PROJECT (_project);
-	cb = g_new0 (GbfAmBuildCallback, 1);
-	cb->callback = callback;
-	cb->user_data = user_data;
-
-	project->callbacks = g_list_append (project->callbacks, cb);
-}
-
-static void
-impl_remove_build_callback (GbfProject      *_project,
-			    GbfBuildCallback callback,
-			    GError         **error)
-{
-	GbfAmProject *project;
-	GbfAmBuildCallback *cb;
-	GList *l;
-
-	g_return_if_fail (GBF_IS_AM_PROJECT (_project));
-
-	project = GBF_AM_PROJECT (_project);
-	l = g_list_find_custom (project->callbacks, callback, find_callback);
-	cb = l->data;
-	project->callbacks = g_list_delete_link (project->callbacks, l);
-	g_free (cb);
+	g_return_val_if_fail (GBF_IS_AM_PROJECT (_project),
+			      GBF_PROJECT_CAN_ADD_NONE);
+	return (GBF_PROJECT_CAN_ADD_GROUP |
+		GBF_PROJECT_CAN_ADD_TARGET |
+		GBF_PROJECT_CAN_ADD_SOURCE);
 }
 
 static GbfProjectGroup * 
@@ -2990,6 +2801,29 @@ impl_add_group (GbfProject  *_project,
 
 	project = GBF_AM_PROJECT (_project);
 	
+	/* Validate group name */
+	if (!name || strlen (name) <= 0)
+	{
+		error_set (error, GBF_PROJECT_ERROR_VALIDATION_FAILED,
+			   _("Please specify group name"));
+		return NULL;
+	}
+	{
+		gboolean failed = FALSE;
+		const gchar *ptr = name;
+		while (*ptr) {
+			if (!isalnum (*ptr) && *ptr != '.' && *ptr != '-' &&
+			    *ptr != '_')
+				failed = TRUE;
+			ptr++;
+		}
+		if (failed) {
+			error_set (error, GBF_PROJECT_ERROR_VALIDATION_FAILED,
+				   _("Group name can only contain alphanumeric, '_', '-' or '.' characters"));
+			return NULL;
+		}
+	}
+	
 	/* find the parent group */
 	g_node = g_hash_table_lookup (project->groups, parent_id);
 	if (g_node == NULL) {
@@ -3208,6 +3042,8 @@ impl_add_target (GbfProject  *_project,
 	gchar *retval;
 
 	g_return_val_if_fail (GBF_IS_AM_PROJECT (_project), NULL);
+	g_return_val_if_fail (type != NULL, NULL);
+	g_return_val_if_fail (name != NULL, NULL);
 
 	project = GBF_AM_PROJECT (_project);
 	
@@ -3218,16 +3054,55 @@ impl_add_target (GbfProject  *_project,
 			   _("Group doesn't exist"));
 		return NULL;
 	}
-
-	/* FIXME: maybe pre-check the target type? */
-
+	
+	/* Validate target name */
+	if (!name || strlen (name) <= 0)
+	{
+		error_set (error, GBF_PROJECT_ERROR_VALIDATION_FAILED,
+			   _("Please specify target name"));
+		return NULL;
+	}
+	{
+		gboolean failed = FALSE;
+		const gchar *ptr = name;
+		while (*ptr) {
+			if (!isalnum (*ptr) && *ptr != '.' && *ptr != '-' &&
+			    *ptr != '_')
+				failed = TRUE;
+			ptr++;
+		}
+		if (failed) {
+			error_set (error, GBF_PROJECT_ERROR_VALIDATION_FAILED,
+				   _("Target name can only contain alphanumeric, '_', '-' or '.' characters"));
+			return NULL;
+		}
+	}
+	if (!strcmp (type, "shared_lib")) {
+		if (strlen (name) < 7 ||
+		    strncmp (name, "lib", strlen("lib")) != 0 ||
+		    strcmp (&name[strlen(name) - 3], ".la") != 0) {
+			error_set (error, GBF_PROJECT_ERROR_VALIDATION_FAILED,
+				   _("Shared library target name must be of the form 'libxxx.la'"));
+			return NULL;
+		}
+	}
+	else if (!strcmp (type, "static_lib")) {
+		if (strlen (name) < 6 ||
+		    strncmp (name, "lib", strlen("lib")) != 0 ||
+		    strcmp (&name[strlen(name) - 2], ".a") != 0) {
+			error_set (error, GBF_PROJECT_ERROR_VALIDATION_FAILED,
+				   _("Static library target name must be of the form 'libxxx.a'"));
+			return NULL;
+		}
+	}
+	
 	/* check that the target doesn't already exist */
 	iter_node = g_node_first_child (g_node);
 	while (iter_node) {
 		GbfAmNode *node = GBF_AM_NODE (iter_node);
 		if (node->type == GBF_AM_NODE_TARGET &&
 		    !strcmp (node->name, name)) {
-			error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
+			error_set (error, GBF_PROJECT_ERROR_ALREADY_EXISTS,
 				   _("Target already exists"));
 			return NULL;
 		}
@@ -3237,8 +3112,8 @@ impl_add_target (GbfProject  *_project,
 	/* Create the update xml */
 	doc = xml_new_change_doc (project);
 	if (!xml_write_add_target (project, doc, g_node, name, type)) {
-		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
-			   _("Target couldn't be created"));
+		error_set (error, GBF_PROJECT_ERROR_GENERAL_FAILURE,
+			   _("General failure in target creation"));
 		xmlFreeDoc (doc);
 		return NULL;
 	}
@@ -3260,12 +3135,13 @@ impl_add_target (GbfProject  *_project,
 	/* get newly created target id */
 	retval = NULL;
 	DEBUG (change_set_debug_print (change_set));
-	change = change_set_find (change_set, GBF_AM_CHANGE_ADDED, GBF_AM_NODE_TARGET);
+	change = change_set_find (change_set, GBF_AM_CHANGE_ADDED,
+				  GBF_AM_NODE_TARGET);
 	if (change) {
 		retval = g_strdup (change->id);
 	} else {
 		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
-			   _("Target couldn't be created"));
+			   _("Newly created target could not be identified"));
 	}
 	change_set_destroy (change_set);
 
@@ -3325,6 +3201,8 @@ impl_name_for_type (GbfProject  *_project,
 		return _("Static Library");
 	} else if (!strcmp (type, "shared_lib")) {
 		return _("Shared Library");
+	} else if (!strcmp (type, "headers")) {
+		return _("Header Files");
 	} else if (!strcmp (type, "man")) {
 		return _("Man Documentation");
 	} else if (!strcmp (type, "data")) {
@@ -3349,17 +3227,19 @@ impl_mimetype_for_type (GbfProject  *_project,
 			const gchar *type)
 {
 	if (!strcmp (type, "static_lib")) {
-		return "application/x-unix-archive";
+		return "application/x-archive";
 	} else if (!strcmp (type, "shared_lib")) {
-		return "application/x-shared-library";
+		return "application/x-sharedlib";
+	} else if (!strcmp (type, "headers")) {
+		return "text/x-chdr";
 	} else if (!strcmp (type, "man")) {
 		return "text/x-troff-man";
 	} else if (!strcmp (type, "data")) {
 		return "application/octet-stream";
 	} else if (!strcmp (type, "program")) {
-		return "application/x-executable-binary";
+		return "application/x-executable";
 	} else if (!strcmp (type, "script")) {
-		return "text/x-sh";
+		return "text/x-shellscript";
 	} else if (!strcmp (type, "info")) {
 		return "application/x-tex-info";
 	} else if (!strcmp (type, "java")) {
@@ -3374,7 +3254,7 @@ impl_mimetype_for_type (GbfProject  *_project,
 static gchar **
 impl_get_types (GbfProject *_project)
 {
-	return g_strsplit ("program:shared_lib:static_lib:"
+	return g_strsplit ("program:shared_lib:static_lib:headers:"
 			   "man:data:script:info:java:python", ":", 0);
 }
 
@@ -3475,11 +3355,33 @@ impl_add_source (GbfProject  *_project,
 	GSList *change_set = NULL;
 	GbfAmChange *change;
 	gchar *retval;
+	gchar *filename;
+	const gchar *ptr;
+	gboolean failed = FALSE;
 	
 	g_return_val_if_fail (GBF_IS_AM_PROJECT (_project), NULL);
-
+	g_return_val_if_fail (uri != NULL, NULL);
+	g_return_val_if_fail (target_id != NULL, NULL);
+	
 	project = GBF_AM_PROJECT (_project);
 	
+	filename = g_path_get_basename (uri);
+	
+	/* Validate target name */
+	ptr = filename;
+	while (*ptr) {
+		if (!isalnum (*ptr) && *ptr != '.' && *ptr != '-' &&
+		    *ptr != '_')
+			failed = TRUE;
+		ptr++;
+	}
+	if (failed) {
+		error_set (error, GBF_PROJECT_ERROR_VALIDATION_FAILED,
+			   _("Source file name can only contain alphanumeric, '_', '-' or '.' characters"));
+		g_free (filename);
+		return NULL;
+	}
+	
 	/* check target */
 	g_node = g_hash_table_lookup (project->targets, target_id);
 	if (g_node == NULL) {
@@ -3493,16 +3395,54 @@ impl_add_source (GbfProject  *_project,
 	 * first */
 	group_uri = uri_normalize (g_path_skip_root (GBF_AM_NODE (g_node->parent)->id),
 				   project->project_root_uri);
+	
 	full_uri = uri_normalize (uri, group_uri);
-	g_free (group_uri);
 	
 	/* Check that the source uri is inside the project root */
 	if (!uri_is_parent (project->project_root_uri, full_uri)) {
-		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
-			   _("Source file must be inside the project directory"));
-		abort_action = TRUE;
+		/* If not, copy it to target's group directory */
+		GnomeVFSURI *group_vuri, *dest_vuri, *src_vuri;
+		GnomeVFSResult res;
+		
+		src_vuri = gnome_vfs_uri_new (uri);
+		group_vuri = gnome_vfs_uri_new (group_uri);
+		dest_vuri = gnome_vfs_uri_append_file_name (group_vuri,
+							    filename);
+		
+		res = gnome_vfs_xfer_uri (src_vuri, dest_vuri,
+					  GNOME_VFS_XFER_DEFAULT,
+					  GNOME_VFS_XFER_ERROR_MODE_ABORT,
+					  GNOME_VFS_XFER_OVERWRITE_MODE_ABORT,
+					  NULL, NULL);
+		if (res != GNOME_VFS_OK && res != GNOME_VFS_ERROR_FILE_EXISTS) {
+			GbfProjectError gbf_err;
+			gchar *error_str =
+				g_strdup_printf ("Failed to copy source file inside project: %s",
+						 gnome_vfs_result_to_string (res));
+			switch (res) {
+			case GNOME_VFS_ERROR_NOT_FOUND:
+				gbf_err = GBF_PROJECT_ERROR_DOESNT_EXIST;
+				break;
+			default:
+				gbf_err = GBF_PROJECT_ERROR_GENERAL_FAILURE;
+				break;
+			}
+			error_set (error, gbf_err, error_str);
+			g_free (error_str);
+			abort_action = TRUE;
+		} else {
+			g_free (full_uri);
+			full_uri = gnome_vfs_uri_to_string (dest_vuri,
+							    GNOME_VFS_URI_HIDE_NONE);
+		}
+		gnome_vfs_uri_unref (src_vuri);
+		gnome_vfs_uri_unref (group_vuri);
+		gnome_vfs_uri_unref (dest_vuri);
 	}
 	
+	g_free (group_uri);
+	g_free (filename);
+	
 	/* check for source duplicates */
 	iter_node = g_node_first_child (g_node);
 	while (!abort_action && iter_node) {
@@ -3510,8 +3450,8 @@ impl_add_source (GbfProject  *_project,
 		
 		if (node->type == GBF_AM_NODE_SOURCE &&
 		    uri_is_equal (full_uri, node->uri)) {
-			error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
-				   _("Source is already in target"));
+			error_set (error, GBF_PROJECT_ERROR_ALREADY_EXISTS,
+				   _("Source file is already in given target"));
 			abort_action = TRUE;
 		}
 		iter_node = g_node_next_sibling (iter_node);
@@ -3527,8 +3467,8 @@ impl_add_source (GbfProject  *_project,
 	doc = xml_new_change_doc (project);
 
 	if (!xml_write_add_source (project, doc, g_node, full_uri)) {
-		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
-			   _("Source couldn't be added"));
+		error_set (error, GBF_PROJECT_ERROR_GENERAL_FAILURE,
+			   _("General failure in adding source file"));
 		abort_action = TRUE;
 	}
 
@@ -3555,15 +3495,15 @@ impl_add_source (GbfProject  *_project,
 	/* get newly created source id */
 	retval = NULL;
 	DEBUG (change_set_debug_print (change_set));
-	change = change_set_find (change_set, GBF_AM_CHANGE_ADDED, GBF_AM_NODE_SOURCE);
+	change = change_set_find (change_set, GBF_AM_CHANGE_ADDED,
+				  GBF_AM_NODE_SOURCE);
 	if (change) {
 		retval = g_strdup (change->id);
 	} else {
-		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
-			   _("Source couldn't be added"));
+		error_set (error, GBF_PROJECT_ERROR_GENERAL_FAILURE,
+			   _("Newly added source file could not be identified"));
 	}
 	change_set_destroy (change_set);
-
 	return retval;
 }
 
@@ -3628,6 +3568,102 @@ impl_configure (GbfProject *_project, GError **error)
 	return wid;
 }
 
+static GList *
+impl_get_config_modules   (GbfProject *project, GError **error)
+{
+	GbfAmConfigMapping *config;
+	GbfAmConfigValue *value;
+	GError* err = NULL;
+	const gchar* pkg_modules;
+	GList* result = NULL;
+	
+	config = gbf_am_project_get_config (GBF_AM_PROJECT(project), &err);
+	if (err) 
+	{
+		g_propagate_error (error, err);
+		return NULL;
+	}
+	if ((value = gbf_am_config_mapping_lookup (config, "pkg_check_modules")) &&
+	    (pkg_modules = gbf_am_config_value_get_string (value)))
+	{
+		gchar **module;
+		gchar **modules = g_strsplit (pkg_modules, ", ", -1);
+		for (module = modules; *module != NULL; ++module) 
+		{
+			result = g_list_prepend (result, g_strdup (*module));
+		}
+		g_strfreev (modules);
+	}
+	return result;
+}
+
+static gboolean
+package_is_valid (const gchar* package)
+{
+	const gchar* c = package;
+	while (*c != '\0')
+	{
+		if (!g_ascii_isalnum (*c) &&
+		    (*c != '_') && (*c != '-') && (*c != '.') && (*c != '+'))
+		{
+			return FALSE;
+		}
+		c++;
+	}
+	return TRUE;
+}
+
+static GList *
+impl_get_config_packages  (GbfProject *project,
+			   const gchar* module,
+			   GError **error)
+{
+	GbfAmConfigMapping *config;
+	GbfAmConfigValue *module_info;
+	GbfAmConfigMapping *module_info_map;
+	GError* err = NULL;
+	GList* result = NULL;
+	
+	config = gbf_am_project_get_config (GBF_AM_PROJECT(project), &err);
+	if (err) 
+	{
+		g_propagate_error (error, err);
+		return NULL;
+	}
+			
+	gchar *module_key = g_strconcat ("pkg_check_modules_",
+					 module,
+					 NULL);
+	
+	if ((module_info = gbf_am_config_mapping_lookup (config, module_key)) &&
+	    (module_info_map = gbf_am_config_value_get_mapping (module_info))) 
+	{
+		GbfAmConfigValue *pkgs_val;
+		const gchar *packages;
+		
+		if ((pkgs_val = gbf_am_config_mapping_lookup (module_info_map, "packages")) &&
+		    (packages = gbf_am_config_value_get_string (pkgs_val))) 
+		{
+			gchar **pkgs, **pkg;
+			pkgs = g_strsplit (packages, ", ", -1);
+			for (pkg = pkgs; *pkg != NULL; ++pkg) 
+			{
+				gchar* version;
+				if ((version = strchr (*pkg, ' ')))
+					*version = '\0';
+				if (package_is_valid (*pkg))
+				{
+					result = g_list_append (result, 
+								g_strdup (*pkg));
+				}
+			}			
+			g_strfreev (pkgs);
+		}		
+	}
+	g_free (module_key);
+	return result;
+}
+
 static void
 gbf_am_project_class_init (GbfAmProjectClass *klass)
 {
@@ -3642,14 +3678,9 @@ gbf_am_project_class_init (GbfAmProjectClass *klass)
 	object_class->get_property = gbf_am_project_get_property;
 
 	project_class->load = impl_load;
-	project_class->build = impl_build;
-	project_class->clean = impl_clean;
 	project_class->probe = impl_probe;
 	project_class->refresh = impl_refresh;
-
-	project_class->get_build_targets = impl_get_build_targets;
-	project_class->add_build_callback = impl_add_build_callback;
-	project_class->remove_build_callback = impl_remove_build_callback;
+	project_class->get_capabilities = impl_get_capabilities;
 
 	project_class->add_group = impl_add_group;
 	project_class->remove_group = impl_remove_group;
@@ -3673,6 +3704,9 @@ gbf_am_project_class_init (GbfAmProjectClass *klass)
 	project_class->configure_new_source = impl_configure_new_source;
 
 	project_class->configure = impl_configure;
+	project_class->get_config_modules = impl_get_config_modules;
+	project_class->get_config_packages = impl_get_config_packages;
+	
 	project_class->name_for_type = impl_name_for_type;
 	project_class->mimetype_for_type = impl_mimetype_for_type;
 	project_class->get_types = impl_get_types;
@@ -3726,11 +3760,6 @@ gbf_am_project_dispose (GObject *object)
 	g_free (project->project_root_uri);
 	project->project_root_uri = NULL;
 
-	/* operations queue */
-	queue_destroy (project);
-
-	/* disconnect callbacks */
-	callbacks_destroy (project);
 	g_free (project->make_command);
 	g_free (project->configure_command);
 	g_free (project->autogen_command);
