@@ -30,19 +30,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtktreestore.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkexpander.h>
+#include <gtk/gtk.h>
 #include <glade/glade-xml.h>
 
 #include "gbf-i18n.h"
@@ -1007,6 +995,250 @@ gbf_am_properties_get_widget (GbfAmProject *project, GError **error)
 	return top_level;
 }
 
+enum
+{
+	COLUMN_CHECK,
+	COLUMN_MODULE_NAME
+};
+
+static const gchar* get_libs_key(GbfProjectTarget *target)
+{
+	g_return_val_if_fail (target != NULL, "ldadd");
+	/* We need to care here if it is ldadd or libadd #476315 */
+	if (g_str_equal (target->type, "shared_lib"))
+	{
+		return "libadd";
+	}
+	else
+	{
+		return "ldadd";
+	}
+}
+
+static void
+on_module_activate (GtkCellRendererToggle* cell_renderer,
+		    const gchar* tree_path,
+		    GtkTreeView* tree)
+{
+	GtkTreeIter iter;
+	GtkTreeModel* model = gtk_tree_view_get_model (tree);
+	gchar* module_name;
+	gchar* module_cflags;
+	gchar* module_libs;
+	gboolean use;
+	GtkTreePath* path = gtk_tree_path_new_from_string (tree_path);
+	GbfProjectTarget* target = g_object_get_data (G_OBJECT(tree), "target");
+	GbfAmConfigMapping* config = g_object_get_data (G_OBJECT(tree), "config");
+	GbfAmConfigMapping* group_config = g_object_get_data (G_OBJECT(tree), "group_config");	
+	GbfAmConfigValue* am_cpp_value = gbf_am_config_mapping_lookup(group_config, "amcppflags");
+	GbfAmConfigValue* cpp_value = gbf_am_config_mapping_lookup(config, "cppflags");
+	GbfAmConfigValue* libs_value = gbf_am_config_mapping_lookup(config, 
+								    get_libs_key(target));
+	
+	if (!cpp_value)
+	{
+		cpp_value = gbf_am_config_value_new (GBF_AM_TYPE_STRING);
+	}
+	if (!libs_value)
+	{
+		libs_value = gbf_am_config_value_new (GBF_AM_TYPE_STRING);
+	}
+	if (!am_cpp_value)
+	{
+		am_cpp_value = gbf_am_config_value_new (GBF_AM_TYPE_STRING);
+	}
+	
+	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_model_get (model, &iter,
+			    COLUMN_MODULE_NAME, &module_name,
+			    COLUMN_CHECK, &use,
+			    -1);
+	
+	module_cflags = g_strdup_printf ("$(%s_CFLAGS)", (gchar*) module_name);
+	module_libs = g_strdup_printf ("$(%s_LIBS)", (gchar*) module_name);
+	g_free (module_name);
+	gtk_tree_path_free (path);
+	
+	if (!use)
+	{
+		GString* cppflags;
+		GString* am_cpp_flags;
+		GString* libs;
+		cppflags = g_string_new (gbf_am_config_value_get_string (cpp_value));
+		am_cpp_flags = g_string_new (gbf_am_config_value_get_string (am_cpp_value));
+		libs = g_string_new (gbf_am_config_value_get_string (libs_value));
+
+		if (strlen (cppflags->str) && !strstr (cppflags->str, module_cflags))
+		{
+			g_string_append_printf (cppflags, " %s", module_cflags);
+			gbf_am_config_value_set_string (cpp_value, cppflags->str);
+			gbf_am_config_mapping_insert (config, "cppflags", cpp_value);
+		}
+		else if (!strstr (am_cpp_flags->str, module_cflags))
+		{
+			g_string_append_printf (am_cpp_flags, " %s", module_cflags);
+			gbf_am_config_value_set_string (am_cpp_value, am_cpp_flags->str);
+			gbf_am_config_mapping_insert (group_config, "amcppflags", am_cpp_value);
+		}
+		if (!strstr (libs->str, module_libs))
+		{
+			g_string_append_printf (libs, " %s", module_libs);		
+			gbf_am_config_value_set_string (libs_value, libs->str);
+			gbf_am_config_mapping_insert (config, get_libs_key (target), libs_value);
+		}
+
+		g_string_free (libs, TRUE);
+		g_string_free (cppflags, TRUE);
+		g_string_free (am_cpp_flags, TRUE);
+		
+	}
+	else
+	{
+		const gchar* cpp_flags = cpp_value ? gbf_am_config_value_get_string(cpp_value) : NULL;
+		const gchar* am_cpp_flags = am_cpp_value ? gbf_am_config_value_get_string(am_cpp_value) : NULL;
+		const gchar* libs_flags = libs_value ? gbf_am_config_value_get_string(libs_value) : NULL;
+
+		if (cpp_flags && strlen (cpp_flags))
+		{
+			const gchar* start = strstr (cpp_flags, module_cflags);
+			GString* str = g_string_new (cpp_flags);
+			if (start)
+				g_string_erase (str, start - cpp_flags, strlen (module_cflags));
+			gbf_am_config_value_set_string (cpp_value, str->str);
+			g_string_free (str, TRUE);
+			gbf_am_config_mapping_insert (config, "cppflags", cpp_value);
+		}
+		else if (am_cpp_flags)
+		{
+			const gchar* start = strstr (am_cpp_flags, module_cflags);
+			GString* str = g_string_new (am_cpp_flags);
+			if (start)
+				g_string_erase (str, start - am_cpp_flags, strlen (module_cflags));
+			gbf_am_config_value_set_string (am_cpp_value, str->str);
+			g_string_free (str, TRUE);
+			gbf_am_config_mapping_insert (group_config, "amcppflags", am_cpp_value);
+		}
+		if (libs_flags)
+		{
+			const gchar* start = strstr (libs_flags, module_libs);
+			GString* str = g_string_new (libs_flags);
+			if (start)
+				g_string_erase (str, start - libs_flags, strlen (module_libs));
+			gbf_am_config_value_set_string (libs_value, str->str);
+			g_string_free (str, TRUE);
+			gbf_am_config_mapping_insert (config, get_libs_key (target), libs_value);
+		}
+	}
+	gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_CHECK, !use, -1);
+	g_free (module_cflags);
+	g_free (module_libs);	
+}
+
+static gboolean
+update_tree_foreach (GtkTreeModel* model,
+		     GtkTreePath* path,
+		     GtkTreeIter* iter,
+		     GtkWidget* view)
+{
+	gchar* module;
+	gboolean use = FALSE;
+	GbfAmConfigMapping* config = g_object_get_data (G_OBJECT(view), "config");
+	GbfAmConfigMapping* group_config = g_object_get_data (G_OBJECT(view), "group_config");
+	GbfProjectTarget* target = g_object_get_data (G_OBJECT(view), "target");
+	GbfAmConfigValue* am_cpp_value = gbf_am_config_mapping_lookup(group_config, "amcppflags");
+	GbfAmConfigValue* cpp_value = gbf_am_config_mapping_lookup(config, "cppflags");
+	GbfAmConfigValue* libs_value = gbf_am_config_mapping_lookup(config, get_libs_key(target));
+	const gchar* cpp_flags = cpp_value ? gbf_am_config_value_get_string(cpp_value) : NULL;
+	const gchar* am_cpp_flags = am_cpp_value ? gbf_am_config_value_get_string(am_cpp_value) : NULL;
+	const gchar* libs_flags = libs_value ? gbf_am_config_value_get_string(libs_value) : NULL;
+
+	gchar* config_cflags; 
+	gchar* config_libs;
+	
+	gtk_tree_model_get (model, iter,
+			    COLUMN_MODULE_NAME, &module,
+			    -1);
+	
+	config_cflags = g_strdup_printf ("$(%s_CFLAGS)", (gchar*) module);
+	config_libs = g_strdup_printf ("$(%s_LIBS)", (gchar*) module);
+	g_free (module);
+	
+	if ((cpp_flags && strstr (cpp_flags, config_cflags)) || 
+	    (am_cpp_flags && strstr (am_cpp_flags, config_cflags)))
+	{
+		if (libs_flags && strstr(libs_flags, config_libs))
+			use = TRUE;
+	}
+	
+	gtk_list_store_set (GTK_LIST_STORE(model), iter,
+			    COLUMN_CHECK, use,
+			    -1);
+	g_free (config_cflags);
+	g_free (config_libs);
+	
+	return FALSE; /* continue iteration */
+}
+
+static GtkWidget*
+create_module_list (GbfAmProject *project,
+		    GbfProjectTarget *target,
+		    GbfAmConfigMapping *config,
+		    GbfAmConfigMapping *group_config)
+{
+	GtkWidget* view;
+	GtkListStore* list;
+	GList* modules;
+	GList* node;
+	GtkCellRenderer* renderer_text;
+	GtkCellRenderer* renderer_toggle;
+	GtkTreeViewColumn* column_text;
+	GtkTreeViewColumn* column_toggle;
+	
+	g_return_val_if_fail (GBF_IS_AM_PROJECT (project), NULL);
+	
+	list = gtk_list_store_new (2, G_TYPE_BOOLEAN, G_TYPE_STRING);
+	view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(list));
+	g_object_set_data (G_OBJECT(view), "config", config);
+	g_object_set_data (G_OBJECT(view), "group_config", group_config);
+	g_object_set_data (G_OBJECT(view), "target", target);	
+	renderer_text = gtk_cell_renderer_text_new();
+	renderer_toggle = gtk_cell_renderer_toggle_new();
+	g_signal_connect (renderer_toggle, "toggled", G_CALLBACK (on_module_activate),
+			  view);
+	
+	column_toggle = gtk_tree_view_column_new_with_attributes (_("Use"),
+								  renderer_toggle,
+								  "active", COLUMN_CHECK,
+								  NULL);
+	
+	column_text = gtk_tree_view_column_new_with_attributes (_("Module"),
+								renderer_text,
+								"text", COLUMN_MODULE_NAME,
+								NULL);
+	
+	gtk_tree_view_append_column (GTK_TREE_VIEW (view), column_toggle);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (view), column_text);
+	gtk_widget_set_size_request (view, -1, 200);
+	
+	modules = gbf_project_get_config_modules (GBF_PROJECT (project), NULL);
+	
+	for (node = modules; node != NULL; node = g_list_next (node))
+	{
+		GtkTreeIter iter;		
+		
+		gtk_list_store_append (list, &iter);
+		gtk_list_store_set (list, &iter,
+				    COLUMN_CHECK, FALSE,
+				    COLUMN_MODULE_NAME, node->data,
+				    -1);
+	}
+	gtk_tree_model_foreach (GTK_TREE_MODEL(list),
+				(GtkTreeModelForeachFunc) update_tree_foreach,
+				view);
+	
+	return view;
+}
+
 static void
 on_group_widget_destroy (GtkWidget *wid, GtkWidget *table)
 {
@@ -1067,14 +1299,10 @@ gbf_am_properties_get_group_widget (GbfAmProject *project,
 	/* Group name */
 	add_configure_property (project, config, GBF_AM_CONFIG_LABEL,
 				_("Group name:"), group->name, NULL, table, 0);
-	/* Includes */
-	add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
-				_("Includes:"), NULL, "includes",
-				table, 1);
 	/* CFlags */
-	table_cflags = gtk_table_new(6,2, FALSE);
-	expander_cflags = gtk_expander_new(_("Compiler Settings"));
-	gtk_table_attach (GTK_TABLE (table), expander_cflags, 0, 2, 3, 4,
+	table_cflags = gtk_table_new(7,2, FALSE);
+	expander_cflags = gtk_expander_new(_("Advanced"));
+	gtk_table_attach (GTK_TABLE (table), expander_cflags, 0, 2, 2, 3,
 				  GTK_FILL | GTK_EXPAND, GTK_FILL, 5, 3);
 	gtk_container_add(GTK_CONTAINER(expander_cflags), table_cflags);
 	add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
@@ -1089,6 +1317,10 @@ gbf_am_properties_get_group_widget (GbfAmProject *project,
 				_("Java compiler flags (just-in-time):"), NULL, "amjavaflags", table_cflags, 4);
 	add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
 				_("Fortran compiler flags:"), NULL, "amfflags", table_cflags, 5);
+	/* Includes */
+	add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
+				_("Includes (deprecated):"), NULL, "includes",
+				table_cflags, 6);
 				
 	/* Install directories */
 	value = gbf_am_config_mapping_lookup (config, "installdirs");
@@ -1105,7 +1337,7 @@ gbf_am_properties_get_group_widget (GbfAmProject *project,
 		gtk_frame_set_shadow_type (GTK_FRAME (frame),
 					   GTK_SHADOW_NONE);
 		gtk_widget_show (frame);
-		gtk_table_attach (GTK_TABLE (table), frame, 0, 2, 4, 5,
+		gtk_table_attach (GTK_TABLE (table), frame, 0, 2, 3, 4,
 				  GTK_FILL | GTK_EXPAND, GTK_FILL, 5, 3);
 		table2 = gtk_table_new (0, 0, FALSE);
 		gtk_widget_show (table2);
@@ -1127,15 +1359,89 @@ on_target_widget_destroy (GtkWidget *wid, GtkWidget *table)
 	
 	GbfAmProject *project = g_object_get_data (G_OBJECT (table), "__project");
 	GbfAmConfigMapping *new_config = g_object_get_data (G_OBJECT (table), "__config");
+	GbfAmConfigMapping *new_group_config = g_object_get_data (G_OBJECT (table), "__group_config");
 	const gchar *target_id = g_object_get_data (G_OBJECT (table), "__target_id");
+	const gchar *group_id = g_object_get_data (G_OBJECT (table), "__group_id");
 	gbf_am_project_set_target_config (project, target_id, new_config, &err);
 	if (err) {
 		g_warning (err->message);
 		g_error_free (err);
 	}
+	err = NULL;
+	gbf_am_project_set_group_config (project, group_id, new_group_config, &err);
+	if (err) {
+		g_warning (err->message);
+		g_error_free (err);
+	}
 	g_object_unref (table);
 }
 
+static void
+on_advanced_clicked (GtkButton* button,
+		     GtkWidget* target_widget)
+{
+	GtkWidget* dialog;
+	GtkWidget* table_cflags;
+	const gchar* lib_type;
+	GbfAmProject* project;
+	GbfAmConfigMapping* config;
+	GbfProjectTarget* target;
+	GtkWidget* view;
+	GtkTreeModel* model;
+	
+	project = g_object_get_data (G_OBJECT(target_widget), "__project");
+	config = g_object_get_data (G_OBJECT(target_widget), "__config");
+	target = g_object_get_data (G_OBJECT(target_widget), "__target");
+	view = g_object_get_data (G_OBJECT(target_widget), "__view");
+		
+	table_cflags = gtk_table_new(9,2, FALSE);
+	add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
+				_("C compiler flags:"), NULL, "cflags", table_cflags, 0);
+	add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
+				_("C preprocessor flags"), NULL, "cppflags", table_cflags, 1);
+	add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
+				_("C++ compiler flags"), NULL, "cxxflags", table_cflags, 2);
+	add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
+				_("gcj compiler flags (ahead-of-time)"), NULL, "gcjflags", table_cflags, 3);
+	add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
+				_("Fortran compiler flags:"), NULL, "amfflags", table_cflags, 4);
+	/* LDFLAGS */
+	add_configure_property (project, config,
+				GBF_AM_CONFIG_ENTRY,
+				_("Linker flags:"), NULL,
+				"ldflags", table_cflags, 6);
+	
+	lib_type = get_libs_key (target);			     
+	add_configure_property (project, config,
+				GBF_AM_CONFIG_ENTRY,
+				_("Libraries:"), NULL,
+				lib_type, table_cflags, 7);
+	
+	/* DEPENDENCIES */
+	add_configure_property (project, config,
+				GBF_AM_CONFIG_ENTRY,
+				_("Dependencies:"), NULL,
+				"explicit_deps", table_cflags, 8);
+	
+	dialog = gtk_dialog_new_with_buttons (_("Advanced options"),
+					      NULL,
+					      GTK_DIALOG_MODAL,
+					      GTK_STOCK_CLOSE,
+					      GTK_RESPONSE_CLOSE,
+					      NULL);
+	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table_cflags);
+	gtk_widget_show_all (dialog);
+	
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	
+	/* Little hack to reuse func... */
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
+	gtk_tree_model_foreach (model,
+				(GtkTreeModelForeachFunc) update_tree_foreach,
+				view);
+}
+
 GtkWidget*
 gbf_am_properties_get_target_widget (GbfAmProject *project,
 				     const gchar *target_id, GError **error)
@@ -1148,8 +1454,6 @@ gbf_am_properties_get_target_widget (GbfAmProject *project,
 	GbfAmConfigMapping *installdirs;
 	GbfAmConfigValue *installdirs_val, *dir_val;
 	GtkWidget *table;
-	GtkWidget *table_cflags;
-	GtkWidget *expander_cflags;
 	GError *err = NULL;
 	
 	g_return_val_if_fail (GBF_IS_AM_PROJECT (project), NULL);
@@ -1173,15 +1477,22 @@ gbf_am_properties_get_target_widget (GbfAmProject *project,
 	group_config = gbf_am_project_get_group_config (project,
 							target->group_id,
 							NULL);
-	
 	table = gtk_table_new (9, 2, FALSE);
 	g_object_ref (table);
 	g_object_set_data (G_OBJECT (table), "__project", project);
 	g_object_set_data_full (G_OBJECT (table), "__config", config,
 				(GDestroyNotify)gbf_am_config_mapping_destroy);
+	g_object_set_data_full (G_OBJECT (table), "__group_config", group_config,
+				(GDestroyNotify)gbf_am_config_mapping_destroy);
 	g_object_set_data_full (G_OBJECT (table), "__target_id",
 				g_strdup (target_id),
 				(GDestroyNotify)g_free);
+	g_object_set_data_full (G_OBJECT (table), "__group_id",
+				g_strdup(group->id),
+				(GDestroyNotify)g_free);
+	g_object_set_data_full (G_OBJECT (table), "__target", 
+				target,
+				(GDestroyNotify)gbf_project_target_free);
 	g_signal_connect (table, "destroy",
 			  G_CALLBACK (on_target_widget_destroy), table);
 	
@@ -1246,51 +1557,19 @@ gbf_am_properties_get_target_widget (GbfAmProject *project,
 	if (target->type && (strcmp (target->type, "program") == 0 ||
 			     strcmp (target->type, "shared_lib") == 0 ||
 			     strcmp (target->type, "static_lib") == 0)) {
-		/* CFLAGS */
-		const gchar* lib_type;
-		table_cflags = gtk_table_new(5,2, FALSE);
-		expander_cflags = gtk_expander_new(_("Compiler Settings"));
-		gtk_table_attach (GTK_TABLE (table), expander_cflags, 0, 2, 4, 5,
-				  GTK_FILL | GTK_EXPAND, GTK_FILL, 5, 3);
-		gtk_container_add(GTK_CONTAINER(expander_cflags), table_cflags);
-		add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
-				_("C compiler flags:"), NULL, "cflags", table_cflags, 0);
-		add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
-				_("C preprocessor flags"), NULL, "cppflags", table_cflags, 1);
-		add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
-				_("C++ compiler flags"), NULL, "cxxflags", table_cflags, 2);
-		add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
-				_("gcj compiler flags (ahead-of-time)"), NULL, "gcjflags", table_cflags, 3);
-		add_configure_property (project, config, GBF_AM_CONFIG_ENTRY,
-				_("Fortran compiler flags:"), NULL, "amfflags", table_cflags, 4);
-		/* LDFLAGS */
-		add_configure_property (project, config,
-					GBF_AM_CONFIG_ENTRY,
-					_("Linker flags:"), NULL,
-					"ldflags", table, 6);
-		
-		/* LDADD */
-		/* We need to care here if it is ldadd or libadd #476315 */
-		if (strcmp (target->type, "shared_lib") == 0)
-		{
-			lib_type = "libadd";
-		}
-		else
-		{
-			lib_type = "ldadd";
-		}			     
-		add_configure_property (project, config,
-					GBF_AM_CONFIG_ENTRY,
-					_("Libraries:"), NULL,
-					lib_type, table, 7);
-		
-		/* DEPENDENCIES */
-		add_configure_property (project, config,
-					GBF_AM_CONFIG_ENTRY,
-					_("Dependencies:"), NULL,
-					"explicit_deps", table, 8);
+		GtkWidget* scrolled_window;
+		GtkWidget* view =  create_module_list(project, target, 
+						      config, group_config);
+		GtkWidget* button = gtk_button_new_with_label (_("Advanced..."));
+		scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+		gtk_container_add (GTK_CONTAINER(scrolled_window), view);
+		gtk_table_attach (GTK_TABLE (table), scrolled_window, 0, 2, 4, 5,
+				  GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 5, 3);
+		gtk_table_attach (GTK_TABLE (table), button, 0, 2, 5, 6,
+				  GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 5, 3);	
+		g_object_set_data (G_OBJECT (table), "__view", view);
+		g_signal_connect (button, "clicked", G_CALLBACK(on_advanced_clicked), table);
 	}
 	gtk_widget_show_all (table);
-	gbf_project_target_free (target);
 	return table;
 }
