@@ -46,7 +46,6 @@
 #include <gbf/glue-plugin.h>
 #include "gbf-i18n.h"
 #include "gbf-mkfile-project.h"
-#include "gbf-mkfile-build.h"
 #include "gbf-mkfile-config.h"
 #include "gbf-mkfile-properties.h"
 
@@ -188,13 +187,6 @@ static GbfProject *parent_class;
    Private prototypes
    ---------------------------------------------------------------------- */
 
-#ifdef NATIVE_GNU_REGEX
-static gboolean        queue_check                  (GbfMkfileProject      *project);
-static void            queue_push_op                (GbfMkfileProject      *project,
-						     GbfMkfileProjectOp    *op);
-#endif /* NATIVE_GNU_REGEX */
-static void            queue_destroy                (GbfMkfileProject      *project);
-
 static gboolean        uri_is_equal                 (const gchar       *uri1,
 						     const gchar       *uri2);
 static gboolean        uri_is_parent                (const gchar       *parent_uri,
@@ -282,123 +274,6 @@ static void            gbf_mkfile_project_get_property  (GObject           *obje
 						     GValue            *value,
 						     GParamSpec        *pspec);
 
-
-/*
- * Queue operations ------------------------
- *
- * - Operations are added to the head and retrieved from the tail
- */
-
-#ifdef NATIVE_GNU_REGEX
-
-static gboolean
-queue_check (GbfMkfileProject *project)
-{
-	GbfMkfileProjectOp *op;
-
-	g_return_val_if_fail (project != NULL, FALSE);
-	g_return_val_if_fail (GBF_IS_MKFILE_PROJECT (project), FALSE);
-
-	g_assert (project->queue_ops != NULL);
-	
-	if (g_queue_is_empty (project->queue_ops)) {
-		project->queue_handler_tag = 0;
-		return FALSE;
-	}
-
-	op = g_queue_pop_tail (project->queue_ops);
-
-	switch (op->type) {
-		case BUILD: 
-		{
-			gchar *project_root = uri_to_path (project->project_root_uri);
-						
-			/* FIXME: gbf_build_run can be simpler */
-			gbf_build_run (project, op->build_id,
-				       project_root,
-				       project->callbacks);
-
-			g_free (op->build_id);
-			g_free (project_root);
-			
-			break;
-		}
-	}
-
-	g_free (op);
-
-	/* FIXME: this matches the old behavior, but it's wrong */
-	if (g_queue_is_empty (project->queue_ops)) {
-		project->queue_handler_tag = 0;
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-static void
-queue_push_op (GbfMkfileProject *project, GbfMkfileProjectOp *op)
-{
-	g_return_if_fail (project != NULL);
-	g_return_if_fail (GBF_IS_MKFILE_PROJECT (project));
-	g_return_if_fail (op != NULL);
-	
-	g_assert (project->queue_ops != NULL);
-	
-	g_queue_push_head (project->queue_ops, op);
-
-	if (project->queue_handler_tag == 0)
-		project->queue_handler_tag = g_idle_add ((GSourceFunc) queue_check, project);
-}
-
-#endif /* NATIVE_GNU_REGEX */
-
-static void
-queue_destroy (GbfMkfileProject *project)
-{
-	g_return_if_fail (project != NULL);
-	g_return_if_fail (GBF_IS_MKFILE_PROJECT (project));
-	
-	if (project->queue_ops) {
-		GbfMkfileProjectOp *op;
-		while ((op = g_queue_pop_tail (project->queue_ops)) != NULL)
-			g_free (op);
-		g_queue_free (project->queue_ops);
-		project->queue_ops = NULL;
-	}
-
-	if (project->queue_handler_tag) {
-		g_source_remove (project->queue_handler_tag);
-		project->queue_handler_tag = 0;
-	}
-}
-
-/*
- * Callbacks bookkeepping -----------------------------
- */
-
-static int
-find_callback (gconstpointer a, gconstpointer b)
-{
-	GbfMkfileBuildCallback *cb = (GbfMkfileBuildCallback *)a;
-	if (cb->callback == b)
-		return 0;
-	else
-		return -1;
-}
-
-static void
-callbacks_destroy (GbfMkfileProject *project)
-{
-	g_return_if_fail (project != NULL);
-	g_return_if_fail (GBF_IS_MKFILE_PROJECT (project));
-
-	/* FIXME */
-	g_list_free (project->callbacks);
-	project->callbacks = NULL;
-}
-
-
 /*
  * URI and path manipulation functions -----------------------------
  */
@@ -2677,53 +2552,6 @@ impl_load (GbfProject  *_project,
 	}
 }
 
-static void
-impl_build (GbfProject  *_project,
-	    const gchar *id,
-	    GError     **error)
-{
-#ifdef NATIVE_GNU_REGEX
-	GbfMkfileProject *project;
-	GbfMkfileProjectOp *op;
-
-	g_return_if_fail (GBF_IS_MKFILE_PROJECT (_project));
-
-	project = GBF_MKFILE_PROJECT (_project);
-	op = g_new0 (GbfMkfileProjectOp, 1);
-	op->project = project;
-	op->type = BUILD;
-
-	op->build_id = g_strdup (id);
-	queue_push_op (project, op);
-#else  /* NATIVE_GNU_REGEX */
-	UNIMPLEMENTED;
-        g_warning ("This method has been disabled until glib-2.14 is released");
-#endif /* NATIVE_GNU_REGEX */
-}
-
-static void
-impl_clean (GbfProject *_project,
-	    GError    **error)
-{
-#ifdef NATIVE_GNU_REGEX
-	GbfMkfileProject *project;
-	GbfMkfileProjectOp *op;
-
-	g_return_if_fail (GBF_IS_MKFILE_PROJECT (_project));
-
-	project = GBF_MKFILE_PROJECT (_project);
-	op = g_new0 (GbfMkfileProjectOp, 1);
-	op->project = project;
-	op->type = BUILD;
-	op->build_id = g_strdup (GBF_MKFILE_BUILD_ID_CLEAN);
-
-	queue_push_op (project, op);
-#else  /* NATIVE_GNU_REGEX */
-	UNIMPLEMENTED;
-        g_warning ("This method has been disabled until glib-2.14 is released");
-#endif /* NATIVE_GNU_REGEX */
-}
-
 static gboolean
 file_exists (const gchar *path, const gchar *filename)
 {
@@ -2783,99 +2611,6 @@ impl_get_capabilities (GbfProject *_project, GError    **error)
 	return GBF_PROJECT_CAN_ADD_NONE;
 }
 
-static void
-foreach_build_target (gpointer key, gpointer value, gpointer data)
-{
-	GList **targets = data;
-	GbfMkfileNode *node;
-	GbfBuildTarget *target;
-
-	node = GBF_MKFILE_NODE ((GNode *)value);
-
-	if ( 0 == strcmp (node->detail, "program") ||
-	     0 == strcmp (node->detail, "static_lib") ||
-	     0 == strcmp (node->detail, "shared_lib") ) {
-		target = g_new0 (GbfBuildTarget, 1);
-		target->id = g_strdup_printf ("USER:%s", (gchar *) key);
-		target->label = g_strdup (node->name);
-		target->description = g_strdup_printf (_("Build specific target: %s"), node->name);
-
-		*targets = g_list_append (*targets, target);
-	}
-}
-
-
-static GList *
-impl_get_build_targets (GbfProject *_project,
-			GError    **error)
-{
-	GbfMkfileProject *project;
-	GbfBuildTarget *target;
-	GList *targets = NULL;
-
-	g_return_val_if_fail (GBF_IS_MKFILE_PROJECT (_project), NULL);
-
-	project = GBF_MKFILE_PROJECT (_project);
-
-	/* FIXME: these targets should be gotten from the Makefile.am files */
-
-	/* "make all" */
-	target = g_new0 (GbfBuildTarget, 1);
-	target->id = g_strdup (GBF_MKFILE_BUILD_ID_ALL);
-	target->label = g_strdup (_("All"));
-	target->description = g_strdup (_("Build the entire project"));
-	targets = g_list_append (targets, target);
-
-	/* "make install" */
-	target = g_new0 (GbfBuildTarget, 1);
-	target->id = g_strdup (GBF_MKFILE_BUILD_ID_INSTALL);
-	target->label = g_strdup (_("Install"));
-	target->description = g_strdup (_("Build the entire project and install it"));
-	targets = g_list_append (targets, target);
-
-	/* other build targets */
-	g_hash_table_foreach (project->targets, foreach_build_target, &targets);
-
-	return targets;
-}
-
-static void 
-impl_add_build_callback (GbfProject      *_project,
-			 GbfBuildCallback callback,
-			 gpointer         user_data,
-			 GError         **error)
-{
-	GbfMkfileProject *project;
-	GbfMkfileBuildCallback *cb;
-
-	g_return_if_fail (GBF_IS_MKFILE_PROJECT (_project));
-
-	project = GBF_MKFILE_PROJECT (_project);
-	cb = g_new0 (GbfMkfileBuildCallback, 1);
-	cb->callback = callback;
-	cb->user_data = user_data;
-
-	project->callbacks = g_list_append (project->callbacks, cb);
-}
-
-static void
-impl_remove_build_callback (GbfProject      *_project,
-			    GbfBuildCallback callback,
-			    GError         **error)
-{
-	GbfMkfileProject *project;
-	GbfMkfileBuildCallback *cb;
-	GList *l;
-
-	g_return_if_fail (GBF_IS_MKFILE_PROJECT (_project));
-
-	project = GBF_MKFILE_PROJECT (_project);
-	l = g_list_find_custom (project->callbacks, callback, find_callback);
-	cb = l->data;
-	project->callbacks = g_list_delete_link (project->callbacks, l);
-	g_free (cb);
-}
-
 static GbfProjectGroup * 
 impl_get_group (GbfProject  *_project,
 		const gchar *id,
@@ -3659,16 +3394,10 @@ gbf_mkfile_project_class_init (GbfMkfileProjectClass *klass)
 	object_class->get_property = gbf_mkfile_project_get_property;
 
 	project_class->load = impl_load;
-	project_class->build = impl_build;
-	project_class->clean = impl_clean;
 	project_class->probe = impl_probe;
 	project_class->refresh = impl_refresh;
 	project_class->get_capabilities = impl_get_capabilities;
 
-	project_class->get_build_targets = impl_get_build_targets;
-	project_class->add_build_callback = impl_add_build_callback;
-	project_class->remove_build_callback = impl_remove_build_callback;
-
 	project_class->add_group = impl_add_group;
 	project_class->remove_group = impl_remove_group;
 	project_class->get_group = impl_get_group;
@@ -3747,11 +3476,6 @@ gbf_mkfile_project_dispose (GObject *object)
 	g_free (project->project_root_uri);
 	project->project_root_uri = NULL;
 
-	/* operations queue */
-	queue_destroy (project);
-
-	/* disconnect callbacks */
-	callbacks_destroy (project);
 	g_free (project->make_command);
 	g_free (project->configure_command);
 	g_free (project->autogen_command);
